%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reproducible file accompanying the paper
% OPTIMAL ALGORITHMS FOR COMPUTING AVERAGE TEMPERATURES
% by R. DeVore, S. Foucart, M. Hielsberg, G. Mullendore, G. Petrova, P. Wojtaszczyk
% Written by M. Hielsberg in March 2017
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function daily = load_daily(filename)

% ID         is the station identification code.  Please see "ghcnd-stations.txt"
%            for a complete list of stations and their metadata.
% YEAR       is the year of the record.
% 
% MONTH      is the month of the record.
% 
% ELEMENT    is the element type.   There are five core elements as well as a number
%            of addition elements.  
% 	   
% 	The five core elements are:
% 
%      PRCP = Precipitation (tenths of mm)
%      SNOW = Snowfall (mm)
%      SNWD = Snow depth (mm)
%      TMAX = Maximum temperature (tenths of degrees C)
%      TMIN = Minimum temperature (tenths of degrees C)
% 	   
% 	The other elements are:
% 	   
% 	   ACMC = Average cloudiness midnight to midnight from 30-second 
% 	          ceilometer data (percent)
% 	   ACMH = Average cloudiness midnight to midnight from 
% 	          manual observations (percent)
%      ACSC = Average cloudiness sunrise to sunset from 30-second 
% 	          ceilometer data (percent)
% 	   ACSH = Average cloudiness sunrise to sunset from manual 
% 	          observations (percent)
%      AWDR = Average daily wind direction (degrees)
% 	   AWND = Average daily wind speed (tenths of meters per second)
% 	   DAEV = Number of days included in the multiday evaporation
% 	          total (MDEV)
% 	   DAPR = Number of days included in the multiday precipiation 
% 	          total (MDPR)
%      DASF = Number of days included in the multiday snowfall 
% 	          total (MDSF)		  
% 	   DATN = Number of days included in the multiday minimum temperature 
% 	         (MDTN)
% 	   DATX = Number of days included in the multiday maximum temperature 
% 	          (MDTX)
%      DAWM = Number of days included in the multiday wind movement
% 	          (MDWM)
% 	   DWPR = Number of days with non-zero precipitation included in 
% 	          multiday precipitation total (MDPR)
% 	   EVAP = Evaporation of water from evaporation pan (tenths of mm)
% 	   FMTM = Time of fastest mile or fastest 1-minute wind 
% 	          (hours and minutes, i.e., HHMM)
% 	   FRGB = Base of frozen ground layer (cm)
% 	   FRGT = Top of frozen ground layer (cm)
% 	   FRTH = Thickness of frozen ground layer (cm)
% 	   GAHT = Difference between river and gauge height (cm)
% 	   MDEV = Multiday evaporation total (tenths of mm; use with DAEV)
% 	   MDPR = Multiday precipitation total (tenths of mm; use with DAPR and 
% 	          DWPR, if available)
% 	   MDSF = Multiday snowfall total 
% 	   MDTN = Multiday minimum temperature (tenths of degrees C; use with 
% 	          DATN)
% 	   MDTX = Multiday maximum temperature (tenths of degress C; use with 
% 	          DATX)
% 	   MDWM = Multiday wind movement (km)
%      MNPN = Daily minimum temperature of water in an evaporation pan 
% 	          (tenths of degrees C)
%      MXPN = Daily maximum temperature of water in an evaporation pan 
% 	          (tenths of degrees C)
% 	   PGTM = Peak gust time (hours and minutes, i.e., HHMM)
% 	   PSUN = Daily percent of possible sunshine (percent)
% 	   SN*# = Minimum soil temperature (tenths of degrees C)
% 	          where * corresponds to a code
% 	          for ground cover and # corresponds to a code for soil 
%             depth.  
% 		  
%         Ground cover codes include the following:
% 		  0 = unknown
% 		  1 = grass
% 		  2 = fallow
% 		  3 = bare ground
% 		  4 = brome grass
% 		  5 = sod
% 		  6 = straw multch
% 		  7 = grass muck
% 		  8 = bare muck
% 		  
% 		  Depth codes include the following:
% 		  1 = 5 cm
% 		  2 = 10 cm
% 		  3 = 20 cm
% 		  4 = 50 cm
% 		  5 = 100 cm
% 		  6 = 150 cm
% 		  7 = 180 cm
% 		  
% 	   SX*# = Maximum soil temperature (tenths of degrees C) 
% 	          where * corresponds to a code for ground cover 
% 		      and # corresponds to a code for soil depth. 
% 		      See SN*# for ground cover and depth codes. 
%      TAVG = Average temperature (tenths of degrees C)
% 	          [Note that TAVG from source 'S' corresponds
% 		      to an average for the period ending at
% 		      2400 UTC rather than local midnight]
%      THIC = Thickness of ice on water (tenths of mm)	
%  	   TOBS = Temperature at the time of observation (tenths of degrees C)
% 	   TSUN = Daily total sunshine (minutes)
% 	   WDF1 = Direction of fastest 1-minute wind (degrees)
% 	   WDF2 = Direction of fastest 2-minute wind (degrees)
% 	   WDF5 = Direction of fastest 5-second wind (degrees)
% 	   WDFG = Direction of peak wind gust (degrees)
% 	   WDFI = Direction of highest instantaneous wind (degrees)
% 	   WDFM = Fastest mile wind direction (degrees)
%      WDMV = 24-hour wind movement (km)	   
%      WESD = Water equivalent of snow on the ground (tenths of mm)
% 	   WESF = Water equivalent of snowfall (tenths of mm)
% 	   WSF1 = Fastest 1-minute wind speed (tenths of meters per second)
% 	   WSF2 = Fastest 2-minute wind speed (tenths of meters per second)
% 	   WSF5 = Fastest 5-second wind speed (tenths of meters per second)
% 	   WSFG = Peak gust wind speed (tenths of meters per second)
% 	   WSFI = Highest instantaneous wind speed (tenths of meters per second)
% 	   WSFM = Fastest mile wind speed (tenths of meters per second)
% 	   WT** = Weather Type where ** has one of the following values:
% 	   
%             01 = Fog, ice fog, or freezing fog (may include heavy fog)
%             02 = Heavy fog or heaving freezing fog (not always 
% 		           distinquished from fog)
%             03 = Thunder
%             04 = Ice pellets, sleet, snow pellets, or small hail 
%             05 = Hail (may include small hail)
%             06 = Glaze or rime 
%             07 = Dust, volcanic ash, blowing dust, blowing sand, or 
% 		           blowing obstruction
%             08 = Smoke or haze 
%             09 = Blowing or drifting snow
%             10 = Tornado, waterspout, or funnel cloud 
%             11 = High or damaging winds
%             12 = Blowing spray
%             13 = Mist
%             14 = Drizzle
%             15 = Freezing drizzle 
%             16 = Rain (may include freezing rain, drizzle, and
% 		           freezing drizzle) 
%             17 = Freezing rain 
%             18 = Snow, snow pellets, snow grains, or ice crystals
%             19 = Unknown source of precipitation 
%             21 = Ground fog 
%             22 = Ice fog or freezing fog
% 		  
%      WV** = Weather in the Vicinity where ** has one of the following 
% 	          values:
% 		   
% 		   01 = Fog, ice fog, or freezing fog (may include heavy fog)
% 		   03 = Thunder
% 		   07 = Ash, dust, sand, or other blowing obstruction
% 		   18 = Snow or ice crystals
% 		   20 = Rain or snow shower
% 		   
% VALUE1     is the value on the first day of the month (missing = -9999).
% 
% MFLAG1     is the measurement flag for the first day of the month.  There are
%            ten possible values:
% 
%            Blank = no measurement information applicable
%            B     = precipitation total formed from two 12-hour totals
%            D     = precipitation total formed from four six-hour totals
% 	         H     = represents highest or lowest hourly temperature (TMAX or TMIN) 
% 	                 or the average of hourly values (TAVG)
%            K     = converted from knots 
%            L     = temperature appears to be lagged with respect to reported
%                    hour of observation 
%            O     = converted from oktas 
%            P     = identified as "missing presumed zero" in DSI 3200 and 3206
%            T     = trace of precipitation, snowfall, or snow depth
% 	         W     = converted from 16-point WBAN code (for wind direction)
% 
% QFLAG1     is the quality flag for the first day of the month.  There are 
%            fourteen possible values:
% 
%            Blank = did not fail any quality assurance check
%            D     = failed duplicate check
%            G     = failed gap check
%            I     = failed internal consistency check
%            K     = failed streak/frequent-value check
% 	         L     = failed check on length of multiday period 
%            M     = failed megaconsistency check
%            N     = failed naught check
%            O     = failed climatological outlier check
%            R     = failed lagged range check
%            S     = failed spatial consistency check
%            T     = failed temporal consistency check
%            W     = temperature too warm for snow
%            X     = failed bounds check
% 	         Z     = flagged as a result of an official Datzilla 
% 	                 investigation
% 
% SFLAG1     is the source flag for the first day of the month.  There are 
%            twenty nine possible values (including blank, upper and 
% 	         lower case letters):
% 
%            Blank = No source (i.e., data value missing)
%            0     = U.S. Cooperative Summary of the Day (NCDC DSI-3200)
%            6     = CDMP Cooperative Summary of the Day (NCDC DSI-3206)
%            7     = U.S. Cooperative Summary of the Day -- Transmitted 
% 	                 via WxCoder3 (NCDC DSI-3207)
%            A     = U.S. Automated Surface Observing System (ASOS) 
%                    real-time data (since January 1, 2006)
% 	         a     = Australian data from the Australian Bureau of Meteorology
%            B     = U.S. ASOS data for October 2000-December 2005 (NCDC 
%                    DSI-3211)
% 	         b     = Belarus update
% 	         C     = Environment Canada
% 	         E     = European Climate Assessment and Dataset (Klein Tank 
% 	                 et al., 2002)	   
%            F     = U.S. Fort data 
%            G     = Official Global Climate Observing System (GCOS) or 
%                    other government-supplied data
%            H     = High Plains Regional Climate Center real-time data
%            I     = International collection (non U.S. data received through
% 	                 personal contacts)
%            K     = U.S. Cooperative Summary of the Day data digitized from
% 	                 paper observer forms (from 2011 to present)
%            M     = Monthly METAR Extract (additional ASOS data)
% 	         N     = Community Collaborative Rain, Hail,and Snow (CoCoRaHS)
% 	         Q     = Data from several African countries that had been 
% 	                 "quarantined", that is, withheld from public release
% 		             until permission was granted from the respective 
% 	                 meteorological services
%            R     = NCEI Reference Network Database (Climate Reference Network
% 	                 and Regional Climate Reference Network)
% 	         r     = All-Russian Research Institute of Hydrometeorological 
% 	                 Information-World Data Center
%            S     = Global Summary of the Day (NCDC DSI-9618)
%                    NOTE: "S" values are derived from hourly synoptic reports
%                    exchanged on the Global Telecommunications System (GTS).
%                    Daily values derived in this fashion may differ significantly
%                    from "true" daily data, particularly for precipitation
%                    (i.e., use with caution).
% 	         s     = China Meteorological Administration/National Meteorological Information Center/
% 	                 Climatic Data Center (http://cdc.cma.gov.cn)
%            T     = SNOwpack TELemtry (SNOTEL) data obtained from the U.S. 
% 	                 Department of Agriculture's Natural Resources Conservation Service
% 	         U     = Remote Automatic Weather Station (RAWS) data obtained
% 	                 from the Western Regional Climate Center	   
% 	         u     = Ukraine update	   
% 	         W     = WBAN/ASOS Summary of the Day from NCDC's Integrated 
% 	                 Surface Data (ISD).  
%            X     = U.S. First-Order Summary of the Day (NCDC DSI-3210)
% 	         Z     = Datzilla official additions or replacements 
% 	         z     = Uzbekistan update
% 	   
% 	   When data are available for the same time from more than one source,
% 	   the highest priority source is chosen according to the following
% 	   priority order (from highest to lowest):
% 	   Z,R,0,6,C,X,W,K,7,F,B,M,r,E,z,u,b,s,a,G,Q,I,A,N,T,U,H,S
% 	   
% 	   
% VALUE2     is the value on the second day of the month
% 
% MFLAG2     is the measurement flag for the second day of the month.
% 
% QFLAG2     is the quality flag for the second day of the month.
% 
% SFLAG2     is the source flag for the second day of the month.
% 
% ... and so on through the 31st day of the month.  Note: If the month has less 
% than 31 days, then the remaining variables are set to missing (e.g., for April, 
% VALUE31 = -9999, MFLAG31 = blank, QFLAG31 = blank, SFLAG31 = blank).


    fid = fopen(filename);
    if (fid < 0)
        daily = [];
        warning('File not found');
        return;
    end
    [bytes, num_bytes] = fread(fid, '*char');
    fclose(fid);
    
    num_entries = floor(num_bytes / 270);
    
    ids = {num_entries};
    years = zeros(1, num_entries);
    months = zeros(1, num_entries);
    elements = {num_entries};
    values = zeros(31, num_entries);
    mflags = {31, num_entries};
    qflags = {31, num_entries};
    sflags = {31, num_entries};
    

    idx = 1;
    i = 1;
    while (i < num_bytes)
     
        % ID            1-11   Character
        ids{idx} = bytes(i:(i+10), 1)';
        i = i + 11;
        
        % YEAR         12-15   Integer
        years(idx) = str2double(bytes(i:(i+3), 1));
        i = i + 4;
        
        % MONTH        16-17   Integer
        months(idx) = str2double(bytes(i:(i+1), 1));
        i = i + 2;
        
        % ELEMENT      18-21   Character
        elements{idx} = bytes(i:(i+3), 1)';
        i = i + 4;
        

        for j = 1:31
            % VALUE1       22-26   Integer
            values(j, idx) = str2double(bytes(i:(i+4), 1));
            i = i + 5;
            
            % MFLAG1       27-27   Character
            mflags{j, idx} = bytes(i, 1)';
            i = i + 1;

            % QFLAG1       28-28   Character
            qflags{j, idx} = bytes(i, 1)';
            i = i + 1;
            
            if (qflags{j, idx} ~= ' ')
                values(j, idx) = -9999;
            end

            % SFLAG1       29-29   Character
            sflags{j, idx} = bytes(i, 1)';
            i = i + 1;

        end
        i = i + 1; % skip new line

        idx = idx + 1;
    end
    

    daily.ids = ids;
    daily.years = years;
    daily.months = months;
    daily.elements = elements;
    daily.values = values;
    daily.mflags = mflags;
    daily.qflags = qflags;
    daily.sflags = sflags;
    
end
